#!/bin/bash

while getopts "p:n:d:" opt;
do case $opt in
       p) PASSWD=${OPTARG} ;;
       n) NAME=${OPTARG} ;;
       d) DEV=${OPTARG} ;;
       e) ENC=${OPTARG} ;;
       \?) echo "-$OPTARG is not valid" >&2 && exit ;;
   esac
done

init() {
    [ -z "$NAME" ] && read -p "Hostname: " NAME
    while [ -z "$PASSWD" ] || [ ! "$PASSWD" = "$CHECK" ]; do
        read -sp "Root password: " PASSWD
        echo
        read -sp "Repeat: " CHECK
    done
    # Show some possible disks
    [ -z "$DEV" ] && lsblk -nrpo "name,size,model" && read -p "Provide installation medium (e.g. sda, nvme0n1): " DEV
    [[ "$DEV" =~ sd[a-z] ]] && SUF="1-3" && MODE="SATA"
    [[ "$DEV" =~ nvme[0-9]n[0-9] ]] && SUF="p1-3" && MODE="NVME"
    [ -z "$ENC" ] && read -p "Do you want the root partition to be encrypted (y/n)? " ENC
    [ "$ENC" == "y" ] && ENC=true || ENC=false

    echo "+---------------------+"
    echo "| Archlinux Bootstrap |"
    echo "+---------------------+"
    echo    "HOSTNAME    = $NAME"
    echo    "ROOTPASSWD  = ${PASSWD:0:1}***${PASSWD: -1}"
    echo    "DEVICEPARTS = $DEV$SUF"
    echo    "ENCRYPTION  = $ENC"
    read -p "Do you want to continue with these values (y/n)? " cont
    [ ! "$cont" = "y" ] && unset NAME PASSWD DEV ENC && init
    echo "Let's GOOOO"
}

crypt_prepare_disk() {
    set +e
    cryptsetup open --type plain -d /dev/urandom $1 wipe_me
    dd if=/dev/zero of=/dev/mapper/wipe_me bs=1M status=progress
    cryptsetup close wipe_me
    set -e
}

crypt_create_fs() {
    cryptsetup -y -v luksFormat $1
    cryptsetup open $1 root
    mkfs.ext4 /dev/mapper/root
}

# stop on error
set -e

# initialize important values
init

[ "$ENC" == true ] && crypt_prepare_disk "/dev/$DEV"

# All values set, start bootstrapping
gdisk /dev/$DEV <<EOF
o
y
n


+1G
ef00
n


+16G
8200
n



8304
w
y

EOF

[ $MODE == "NVME" ] && suffix="p" || suffix=""

mkfs.fat -F 32 -n P_EFI "/dev/$DEV$suffix1"
if [ "$ENC" == true ]; then
    crypt_create_fs "/dev/$DEV$suffix3"
    mkfs.ext2 -L cryptswap "/dev/$DEV$suffixp2" 1M # Otherwise label will be lost after reboot
else
    mkfs.ext4 -L P_ROOT "/dev/$DEV$suffix3"
    mkswap -L P_SWAP "/dev/$DEV$suffix2"
fi


ROOT_UUID=$(blkid | grep -Po '/dev/'"$DEV$suffix"'3.* UUID="\K[0-9a-f-]+')
SWAP_UUID=$(blkid | grep -Po '/dev/'"$DEV$suffix"'2.* UUID="\K[0-9a-f-]+')

if [ "$ENC" == true ]; then
    mount /dev/mapper/root /mnt
    KERNEL_OPTIONS='cryptdevice=UUID='"$ROOT_UUID"':root root=/dev/mapper/root'
else
    mount -L P_ROOT /mnt
    KERNEL_OPTIONS='root=LABEL=P_ROOT rw resume=LABEL=P_SWAP'
fi

mkdir -p /mnt/boot
mount -L P_EFI /mnt/boot   # EFI

[ "$ENC" == false ] && swapon -L P_SWAP # swap

sed -i '/## Germany/,+1!d' /etc/pacman.d/mirrorlist

# for server dhcpcd and other programs are not required -> see good2know
pacstrap /mnt base base-devel linux-zen linux-firmware vi dhcpcd wpa_supplicant dialog git netctl curl
genfstab -p /mnt > /mnt/etc/fstab

cat <<EOF > /mnt/root/bootstrap2.sh
#!/bin/bash

if [ $ENC == true ]; then
    sed -i 's/\(^HOOKS.*block\)/\1 encrypt/' /etc/mkinitcpio.conf
    ct_entry="swap LABEL=cryptswap /dev/urandom swap,offset=2048,cipher=aes-xts-plain64,size=512"
    echo $ct_entry >> /etc/crypttab
    echo "/dev/mapper/swap none swap defaults 0 0" >> /etc/fstab
fi

ln -sf /usr/share/zoneinfo/Europe/Berlin /etc/localtime

echo "en_US.UTF-8 UTF-8" > /etc/locale.gen
locale-gen

echo "LANG=en_US.UTF-8" > /etc/locale.conf
echo $NAME > /etc/hostname

mkinitcpio -p linux

pacman -Sy --noconfirm efibootmgr dosfstools gptfdisk

bootctl install
# MAYBE: add intel-ucode images, check for amd
cat <<EEE > /boot/loader/entries/arch-uefi.conf
title	Arch
linux	/vmlinuz-linux-zen
initrd	/initramfs-linux-zen.img
options $KERNEL_OPTIONS
EEE

cat <<EEE > /boot/loader/entries/arch-uefi-fallback.conf
title    Arch Linux Fallback
linux    /vmlinuz-linux-zen
initrd   /initramfs-linux-zen-fallback.img
options $KERNEL_OPTIONS
EEE

cat <<EEE > /boot/loader/loader.conf
default   arch-uefi
timeout   1
EEE

pacman --noconfirm -Sy reflector
reflector --country 'Germany' --sort rate --protocol https --save /etc/pacman.d/mirrorlist

printf "${PASSWD}\n${PASSWD}\n" | passwd root

cd && curl -L https://g.phga.de/phga/fresh/archive/master.tar.gz -o fresh.tar.gz && tar -xzf fresh.tar.gz
# basically a shitty one time job, which does not work, seems like root does not source .bashrc
echo '/root/fresh/fresh.sh' > .bash_profile

exit
EOF

chmod u+x /mnt/root/bootstrap2.sh

arch-chroot /mnt /root/bootstrap2.sh
rm /mnt/root/bootstrap2.sh

# reboot
